from yaml import load, FullLoader
from typing import List, Tuple
from itertools import chain
from pathlib import Path
from .header_template import HEADER
from .code_template import CODE


def load_interfaces(path: str) -> List[str]:
    with open(path, mode='rt', encoding='utf-8') as file:
        return load(file, Loader=FullLoader)['interfaces']


def load_interface(path: str) -> Tuple[str, str, List[str]]:    
    with open(path, mode='rt', encoding='utf-8') as file:
        interface = load(file, Loader=FullLoader)
    return (interface['name'], interface['type'], interface['values'])


def generate(name: str,
             type: str,
             values: List[str],
             working_directory: str,
             encoding: str, 
             indent: str,
             line_ending: str) -> None:
    with Path(working_directory) as path:
        if not path.exists():
            path.mkdir()

    enums = chain(values, ['Count'])
    enum_lines = map(lambda value: f'{indent}{value}', enums)
    enum_values = f',{line_ending}'.join(enum_lines)
    with open(f'{name}.h', mode='wt', encoding=encoding) as file:
        file.write(HEADER.format(name=name, 
                                 type=type, 
                                 enum_values=enum_values))

    string_lines = map(lambda value: f'{indent}"{value}"', values)
    string_values = f',{line_ending}'.join(string_lines)
    with open(f'{name}.cpp', mode='wt', encoding=encoding) as file:
        file.write(CODE.format(name=name, string_values=string_values))



CODE = '''#include "{name}.h"

/*
 * generated code from cpp-enum-class-string-idl
 */

const char* _{name}[] = {{
{string_values}
}};

const char* enum_to_string(const {name} value) {{
    if ({name}::Count == value) {{
        return "";
    }}

    return _{name}[static_cast<int>(value)];
}}
'''



import setuptools


setuptools.setup(
    name="cpp-enum-class-string-idl",
    version="0.0.1",
    license='MIT',
    author="oneofthezombies",
    author_email="hunhoekim@gmail.com",
    description="cpp enum class string idl.",
    long_description=open('README.md').read(),
    long_description_content_type = 'text/markdown',
    url="https://github.com/oneofthezombies/cpp-enum-class-string-idl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)



from argparse import ArgumentParser
from more_itertools import consume
from cpp_enum_class_string_idl import (
    load_interfaces, 
    load_interface, 
    generate
)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('interfaces', help='interface list file path.')
    parser.add_argument('--working-directory', default='.', help='path to be generated.')
    parser.add_argument('--encoding', default='utf-8', help="generated file's encoding.")
    parser.add_argument('--indent', default='    ', help="code value's indent string.")
    parser.add_argument('--line-ending', default='lf', choices=['cr', 'lf', 'crlf'], help="code value's line ending.")
    args = parser.parse_args()

    if args.line_ending == 'crlf':
        line_ending = '\r\n'
    elif args.line_ending == 'lf':
        line_ending = '\n'
    elif args.line_ending == 'cr':
        line_ending = '\r'
    else:
        line_ending = '\n'

    paths = load_interfaces(args.interfaces)
    datas = map(lambda path: load_interface(path), paths)
    generates = map(lambda data: generate(*data, 
                                          working_directory=args.working_directory, 
                                          encoding=args.encoding, 
                                          indent=args.indent,
                                          line_ending=line_ending), datas)
    consume(generates)
